/*

  Gothello v1.0
  using TheBrain10   v1.0 of the brain

  class TheBrain10 's member functions code

*/

#include "BRAIN10.H"

//TheBrain10 member functions

TheBrain10::TheBrain10()
	{
	//set default parameters
	me_id=1; you_id=-1; empty_id=0;
	depth=8;
	eg_empty=12;
	move_order_depth=2;

	score_win = 100000, score_loss= -100000, score_draw=0;
	start_alpha = -999999; start_beta = 999999;

	//some limiting values
	max_depth = 35, max_disks_turned = 20, max_legal_moves = 40;

	// allocate memory for move structures
	used_move_struc = 0;
	start_move_struc = new char[max_depth*max_disks_turned*max_legal_moves];
										// space for move structures
										// 35-max depth; 20-max disks overturned by a move
										// 40-max moves possible in a position
	mem_err = (start_move_struc==0)? true : false;
			
	//set the order of search of depth 2 to be logical
	order[0] = 0; order[1] = 7; order[2] = 56; order[3] = 63;
	order[4] = 2; order[5] = 3; order[6] = 4; order[7] = 5;
	order[8] = 23; order[9] = 31; order[10] = 39; order[11] = 47;
	order[12] = 16; order[13] = 24; order[14] = 32; order[15] = 40;
	order[16] = 58; order[17] = 59; order[18] = 60; order[19] = 61;
	order[20] = 18; order[21] = 21; order[22] = 42; order[23] = 45;
	order[24] = 43; order[25] = 37; order[26] = 20; order[27] = 26;
	order[28] = 34; order[29] = 44; order[30] = 29; order[31] = 19;
	order[32] = 10; order[33] = 11; order[34] = 12; order[35] = 13;
	order[36] = 22; order[37] = 30; order[38] = 38; order[39] = 46;
	order[40] = 17; order[41] = 25; order[42] = 33; order[43] = 41;
	order[44] = 50; order[45] = 51; order[46] = 52; order[47] = 53;
	order[48] = 48; order[49] = 55; order[50] = 15; order[51] = 8;
	order[52] = 57; order[53] = 62; order[54] = 6; order[55] = 1;
	order[56] = 49; order[57] = 54; order[58] = 14; order[59] = 9;


	}

TheBrain10::~TheBrain10()
	{
	delete start_move_struc;
	}

bool TheBrain10::is_mem_err()
	{
	return mem_err;
	}

void TheBrain10::set_depth(int d)	//set depth to be searched
	{
	depth = d;
	}

void TheBrain10::set_endgame_empty(int d)	//set the start of endgame with d squares empty
	{
	eg_empty = d;
	}

void TheBrain10::set_move_order_depth(int d)
	{
	move_order_depth = d;
	}

void TheBrain10::set_id(int me, int you, int empty)
	{		//note that the class internally uses +1,-1,0 respectively
	me_id = me; you_id = you; empty_id = empty;
	}

Score TheBrain10::get_cur_score()
	{
	return cur_score;
	}

inline char* TheBrain10::alloc_move_struc()
	{
	assert(used_move_struc<max_depth-1);
	used_move_struc++;
	return start_move_struc + ( (used_move_struc-1) * max_disks_turned * max_legal_moves);
	}

inline void TheBrain10::dealloc_move_struc()
	{
	assert (used_move_struc>0);
	used_move_struc--;
	}

int TheBrain10::make_move(int* brd)
	{
	count = 0; //the no of leaves evaluated
	//copy the given board into a local array & simultaneously find the total no of disks
	total_disks = 64;
	for (int i=0; i<=63; i++)
		{
		if (brd[i]==empty_id) { board[i]=0; total_disks--; }
		else if (brd[i]==you_id) board[i]=-1;
		else board[i]=+1;
		}

	to_play = 1;  //me, myself
	Score alpha = start_alpha, beta = start_beta;

	move_order_search = false;
		//initially this is true search; not the shallow search initiated by order_moves()
		// to perform move ordering.

	char* move_struc;
	move_struc = alloc_move_struc();

	// Important(the following line): if we have entered the end game, then
	// make depth infinite(60), to search till the end of the game...
	int depth_this_time = (total_disks > 64-eg_empty)? 60 : depth;

	int num_moves = set_up_legal_moves(move_struc);

	if (num_moves==0)  //no moves to play for us!
		{
		char* temp_move_struc;
		int q;

		temp_move_struc = alloc_move_struc();
		to_play=-to_play;
		q = set_up_legal_moves(temp_move_struc);
		to_play=-to_play;
		dealloc_move_struc();

		Score to_return;

		if (q==0)		//game ends!!!!
			{
			int i = disk_difference(to_play);

			if (i>0) to_return = score_win + i;
			else if(i<0) to_return = score_loss - i;
			else to_return = score_draw;
			}
		else		// its just a pass from our side, so evaluate & return the opponent's move
			{
			to_play = -to_play;
			to_return = -evaluate(-beta, -alpha, depth_this_time-1);
			to_play = -to_play;
			}

		dealloc_move_struc();
		cur_score = to_return;
		return 64;    //***** 64 indicates illegal move - here it means no move available
		}

	//order the moves for best pruning
	int move_order[40] = { 0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,
		25,26,27,28,29,30,31,32,33,34,35,36,37,38,39};
	order_moves(num_moves, move_struc, move_order);

	//make and evaluate the tree
	Score v = alpha;
	Score e;
	movetype best_move;
	for (i = 0; i<num_moves; i++)
		{
		play_move(move_struc, move_order[i]);

		to_play=-to_play;
		e = -evaluate(-beta, -v, depth_this_time-1);
		to_play=-to_play;

		undo_move(move_struc, move_order[i]);

		if (e > v) { v = e; best_move = move_order[i]; }
		}
	best_move = move_struc[20*best_move];

	dealloc_move_struc();

	cur_score = v;
	return best_move;
	}

Score TheBrain10::evaluate(Score alpha, Score beta, int height)
	{
	//gotoxy(1,1);
	//printf("%d", height);

	if (height==0)	//we have reached a leaf
		return stat_eval();

	char* move_struc;
	move_struc = alloc_move_struc();

	int num_moves = set_up_legal_moves(move_struc);

	if (num_moves==0)  //no moves to play for us!
		{
		char* temp_move_struc;
		int q;

		temp_move_struc = alloc_move_struc();
		to_play = -to_play;
		q = set_up_legal_moves(temp_move_struc);
		to_play = -to_play;
		dealloc_move_struc();

		Score to_return;

		if (q==0)		//game ends!!!!
			{
			int i = disk_difference(to_play);

			if (i>0) to_return = score_win + i;
			else if(i<0) to_return = score_loss + i;
			else to_return = score_draw;
			}
		else		// its just a pass from our side, so evaluate & return the opponent's move
			{
			to_play = -to_play;
			to_return = -evaluate(-beta, -alpha, height-1);
			to_play = -to_play;
			}

		dealloc_move_struc();
		return to_return;
		}

	//order the moves for best pruning
	int move_order[40] = { 0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,
		25,26,27,28,29,30,31,32,33,34,35,36,37,38,39};
	if ( (move_order_search==false) && (height>=4))
		order_moves(num_moves, move_struc, move_order);

	//make and evaluate the tree
	Score v = alpha;
	Score e;
	for (int i = 0; i<num_moves; i++)
		{
		play_move(move_struc, move_order[i]);

		to_play=-to_play;
		e = -evaluate(-beta, -v, height-1);
		to_play=-to_play;

		undo_move(move_struc, move_order[i]);

		if (e > v) v = e;
		if (v>=beta) break;	//alpha-beta cut-off
		}

	dealloc_move_struc();

	return v;
	}

void TheBrain10::order_moves(int num_moves, char* move_struc, int* move_order)
//searches to a shallow depth of move_order_depth(generally 2) and sorts the moves by the
//evaluations thus returned. Because of good move sorting, alpha-beta is even faster
// one complication is that this function should not go on recursing, this will slow things.
// so, the shallow search initiated by this function is marked by the boolean variable
// move_order_search, which will not let this function be called until true search is resumed
	{
	//now its a true search, not the search initiated by order_moves() itself
	move_order_search=true;
	//printf("Inside order_moves");

	int i,j;

	for (i = 0; i<num_moves; i++)
		{
		play_move(move_struc, i);

		to_play=-to_play;
		eval_table[i] = -evaluate(start_alpha, start_beta, move_order_depth-1);
		to_play=-to_play;

		undo_move(move_struc, i);
		}

	//sort the evaluations
	for (i=0; i<num_moves; i++)
		{
		Score max=-1000000L;
		int max_move;
		for(j=0; j<num_moves; j++)
			if (eval_table[j]>max) { max = eval_table[j]; max_move = j; }
		move_order[i] = max_move;
		eval_table[max_move]=-1000000L;
		}

	move_order_search=false;
	//printf("Outside order_moves");
	}

void TheBrain10::play_move(char* move_struc, int movenum)
	{
	int t;
	for (int i=0; i<=19; i++)
		{
		t = move_struc[movenum*20 + i];
		if (t==100) break;
		board[t] = to_play;
		}
	//assert(t==100);
	total_disks++;
	}

void TheBrain10::undo_move(char* move_struc, int movenum)
	{
	board[move_struc[movenum*20]] = 0;	//make empty the square where the piece was placed

	int t;
	for (int i=1; i<=19; i++)
		{
		t = move_struc[movenum*20 + i];
		if (t==100) break;
		board[t] = -to_play;
		}
	//assert(t==100);
	total_disks--;
	}


Score TheBrain10::stat_eval()
	{
	count++;

	Score sum, sum1=0, sum2=0, sum4=0, sum5=0;
	float a,b,d,e;

	if      (total_disks < 20) { a = 9; b = 2; d = 9; e = 6; }
	else if (total_disks < 30) { a = 9; b = 2; d = 9; e = 6; }
	else if (total_disks < 40) { a = 9; b = 2; d = 9; e = 6; }
	else if (total_disks < 48) { a = 9; b=1.7; d = 8; e = 5; }
	else if (total_disks < 53) { a = 9; b=1.2; d = 7; e = 4; }
	else 							  { a = 9; b = 0; d = 0; e = 0; }

	int t1 = strat_3(to_play), t2 = strat_3(-to_play);
	int t = disk_difference(to_play);

	if ( t1==0 && t2==0)					//game has ended
		{
		if (t>0) return score_win + t;
		else if(t<0) return score_loss - t;
		else return score_draw;
		}

	sum1 = t;
	sum2 = strat_1(to_play) - strat_1(-to_play);
	sum4 = t1 - t2;
	sum5 = strat_4(to_play) - strat_4(-to_play);

	sum = a*sum1 + b*sum2+ d*sum4 + e*sum5;
	return sum;
	}

int TheBrain10::strat_1(id who)  //corner discs COMBINED with CONSECUTIVE
	{                              //edge discs give more power
	static int starts[4] = { 0, 7, 56, 63 };
	static int incs[8] = { 1, 8, -1, 8, 1, -8, -1, -8};
	int i,j;
	int corner;
	int to_return=0;

	for(corner=0; corner<=3; corner++)
		{
		int cell = starts[corner];

		if (board[cell] != who) continue;

		to_return += 100;

		for(i=1; i<=6; i++)
			{
			j = cell + i*incs[corner*2];
			if (board[j] != who) break;
			to_return += (i*8);
			}
		for(i=1; i<=6; i++)
			{
			j = cell + i*incs[corner*2+1];
			if (board[j] != who) break;
			to_return += (i*8);
			}
		}

	return to_return;

	}

int TheBrain10::strat_3(id who) //mobility
	{
	return no_of_legal_moves(who);
	}

int TheBrain10::strat_4(id who) //potential mobility
	{
	int to_return=0;
	id you = -who;

	for(int i=0; i<=63; i++)
		if (board[i] == 0)
			{
			int r = i/8, c = i%8;
			if (r<=5) to_return += ( (board[i+8]==you)? 1 : 0);
			if (r>=2) to_return += ( (board[i-8]==you)? 1 : 0);
			if (c<=5) to_return += ( (board[i+1]==you)? 1 : 0);
			if (c>=2) to_return += ( (board[i-1]==you)? 1 : 0);
			if (r<=5 && c<=5) to_return += ( (board[i+9]==you)? 1 : 0);
			if (r<=5 && c>=2) to_return += ( (board[i+7]==you)? 1 : 0);
			if (r>=2 && c<=5) to_return += ( (board[i-7]==you)? 1 : 0);
			if (r>=2 && c>=2) to_return += ( (board[i-9]==you)? 1 : 0);
			}

	return to_return;
	}

int TheBrain10::no_of_legal_moves(id who)
	{
	int to_return=0;
	for(int i=0; i<=63; i++)
		if ( is_legal(who, i) ) to_return++;

	return to_return;
	}

bool TheBrain10::is_legal(id who, movetype m)
	{
	if (board[m]) return false;

	int r = m/8, c = m%8;
	id you = opp(who);

	if (c<=5)
		if (board[m+1]==you)
			for ( int i=m+2; i%8!=0; i++)
				if (board[i]==who) return true;
				else if (board[i]==0) break;
	if (c>=2)
		if (board[m-1]==you)
			for ( int i=m-2; i%8!=7 && i!=-1; i--)
				if (board[i]==who) return true;
				else if (board[i]==0) break;
	if (r>=2)
		if (board[m-8]==you)
			for ( int i=m-16; i>=0 ; i-=8)
				if (board[i]==who) return true;
				else if (board[i]==0) break;
	if (r<=5)
		if (board[m+8]==you)
			for ( int i=m+16; i<=63 ; i+=8)
				if (board[i]==who) return true;
				else if (board[i]==0) break;
	if (r<=5 && c<=5)
		if (board[m+9]==you)
			for ( int i=m+18; (i<=63) && ((i%8)!=0) ; i+=9)
				if (board[i]==who) return true;
				else if (board[i]==0) break;
	if (r<=5 && c>=2)
		if (board[m+7]==you)
			for ( int i=m+14; (i<=63) && ((i%8)!=7) ; i+=7)
				if (board[i]==who) return true;
				else if (board[i]==0) break;
	if (r>=2 && c<=5)
		if (board[m-7]==you)
			for ( int i=m-14; i>=0 && i%8 != 0 ; i-=7)
				if (board[i]==who) return true;
				else if (board[i]==0) break;
	if (r>=2 && c>=2)
		if (board[m-9]==you)
			for ( int i=m-18; (i>=0) && ((i%8)!=7) ; i-=9)
				if (board[i]==who) return true;
				else if (board[i]==0) break;

	return false;
	}

int TheBrain10::disk_difference(id to_play)
	{
	register int i;
	int me=0, you=0;

	for (i=0; i<=63; i++)
		if (board[i]==to_play) me++; else if (board[i]==-to_play) you++;

	return me-you;
	}


int TheBrain10::set_up_legal_moves(char* move_struc)
//returns no of legal moves and sets up the legal moves in the move structure move_struc
	{

	int move_no = 0;
	id who = to_play, other = -to_play;

	for (int z = 0; z<=60; z++)
		{
		movetype m = order[z];

		move_struc[move_no*20] = m;	//put m on it for trial
		int move_effect = 1;  //it will be overwritten next loop if it
									// does not present itself as legal move

		//begin testing
		register int i,j;
		register int r = m/8, c = m%8;

		if (board[m]) continue;

		if (c<=5)
			if (board[m+1]==other)
				for ( i=m+2; i%8!=0; i++)
					{
					if (board[i]==other) continue;
					if (board[i]==who)
						for (int j=m+1; j!=i; j++)
							{
							move_struc[move_no*20 + move_effect] = j;
							move_effect++;
							}
					break;
					}
		if (c>=2)
			if (board[m-1]==other)
				for ( i=m-2; i%8!=7 && i!=-1; i--)
					{
					if (board[i]==other) continue;
					if (board[i]==who)
						for ( j=m-1; j!=i; j--)
							{
							move_struc[move_no*20 + move_effect] = j;
							move_effect++;
							}
					break;
					}
		if (r>=2)
			if (board[m-8]==other)
				for ( int i=m-16; i>=0 ; i-=8)
					{
					if (board[i]==other) continue;
					if (board[i]==who)
						for (int j=m-8; j!=i; j-=8)
							{
							move_struc[move_no*20 + move_effect] = j;
							move_effect++;
							}
					break;
					}
		if (r<=5)
			if (board[m+8]==other)
				for ( int i=m+16; i<=63 ; i+=8)
					{
					if (board[i]==other) continue;
					if (board[i]==who)
						for (int j=m+8; j!=i; j+=8)
							{
							move_struc[move_no*20 + move_effect] = j;
							move_effect++;
							}
					break;
					}
		if (r<=5 && c<=5)
			if (board[m+9]==other)
				for ( int i=m+18; (i<=63) && ((i%8)!=0) ; i+=9)
					{
					if (board[i]==other) continue;
					if (board[i]==who)
						for (int j=m+9; j!=i; j+=9)
							{
							move_struc[move_no*20 + move_effect] = j;
							move_effect++;
							}
					break;
					}
		if (r<=5 && c>=2)
			if (board[m+7]==other)
				for ( int i=m+14; (i<=63) && ((i%8)!=7) ; i+=7)
					{
					if (board[i]==other) continue;
					if (board[i]==who)
						for (int j=m+7; j!=i; j+=7)
							{
							move_struc[move_no*20 + move_effect] = j;
							move_effect++;
							}
					break;
					}
		if (r>=2 && c<=5)
			if (board[m-7]==other)
				for ( int i=m-14; i>=0 && i%8 != 0 ; i-=7)
					{
					if (board[i]==other) continue;
					if (board[i]==who)
						for (int j=m-7; j!=i; j-=7)
							{
							move_struc[move_no*20 + move_effect] = j;
							move_effect++;
							}
					break;
					}
		if (r>=2 && c>=2)
			if (board[m-9]==other)
				for ( int i=m-18; (i>=0) && ((i%8)!=7) ; i-=9)
					{
					if (board[i]==other) continue;
					if (board[i]==who)
						for (int j=m-9; j!=i; j-=9)
							{
							move_struc[move_no*20 + move_effect] = j;
							move_effect++;
							}
					break;
					}

		if (move_effect>1)
			{
			if (move_effect<20) move_struc[move_no*20 + move_effect] = 100;	//indicate termination
			move_no++;	//set pointer ready for next legal move
			}

		}

	move_struc[move_no*20] = 100;

	ASSERT(move_no<34);
	
	return move_no;

	}
