/*
	Othello Version 2

	Improvements over Version 1
		- Complete restructuring of program. More logical, simple,
		  robust, flexible.
		- Mouse support
		- 2 player game support
		- Uses TheMaster Version 2.0, which is highly improved. (TheMaster is
		  used for making the computer play)

	CopyRight(c) Gaurang Khetan, Pune.
	Date: 18/4/1999

*/
#include <STRING.H>
#include <ASSERT.H>
#include <DOS.H>
#include <GRAPHICS.H>
#include <CONIO.H>
#include <STDIO.H>
#include <STDLIB.H>
#include <MATH.H>
#include <TIME.H>
#include <SYS\TIMEB.H>

// *** check whether this required: extern unsigned _stklen = 8096U;

typedef long signed int score;
typedef signed int movetype;
typedef signed int pos;
typedef signed int id;
typedef unsigned int color;
enum bool { false, true };
enum buttontype { leftbutton, rightbutton };
enum playertype { human,computer};

#include "MOUSE.H"
#include "BRAIN21.H"
#include "OTHELLO.H"	//class defintions

display_class disp;
board_class board;
mouse_manager mouse;

bool mouse_present;
bool sound_output = false;

void play_music(int*);

void main()
	{
	if ( registerfarbgidriver(EGAVGA_driver_far) < 0
				|| registerfarbgifont(gothic_font_far) < 0
					|| registerfarbgifont(triplex_font_far) < 0 )
		{
		printf(" Graphics driver error!");
		getch();
		return;
		}

	disp.init_graphics();
	mouse_present = (mouse.reset() == 1)? true : false;

	while(true)
		{
		game_class game;

		game.intro();

		disp.display_grid();
		board.reset_board();

		if (mouse_present) mouse.display();
		if (mouse_present) mouse.flush();
		game.play_game();

		disp.set_line(0, "Want to play again? (y/n) ");

		char ch;
		do ch = getch(); while (ch!='y' && ch!='Y' && ch!='n' && ch!='N');

		if (ch=='n' || ch=='N')
			{
			disp.set_line(0, "Come back again...");
			getch();
			break;
			}
		}

	disp.end_graphics(); //contains mouse.conceal() command

	}

//class display member functions

display_class::display_class()
	{
	cell_hgt = 35; cell_wdth = 35;
	grd_x = 550 - (cell_wdth * 8);
	grd_y = 25 + (cell_hgt * 8);
	piece_rad = 14;

	x1[0] = 1; y1[0] = 300; x2[0] = 250; y2[0] = 325;
	x1[1] = 1; y1[1] = 200; x2[1] = 250; y2[1] = 225;
	x1[2] = 1; y1[2] = 250; x2[2] = 250; y2[2] = 275;
	x1[3] = 300; y1[3] = 350; x2[3] = 500; y2[3] = 375;

	size[0] = size[1] = size[2] = size[3] = 0;

	num_of_moves = 0;
	show_turning = true;
	indi_pos_moves = true;

	back_col = 1;
	piece_cols[1] = 4; piece_cols[2] = 15;
	grd_back_col = 2; grd_col = 15;
	talk_col[0] = WHITE;  //6;
	talk_col[1] = WHITE;  //CYAN;
	talk_col[2] = WHITE;  //8;
	talk_col[3] = WHITE;  //9;
	}

void display_class::init_graphics()
	{
	int gd = VGA, gm = VGAHI;
	initgraph(&gd, &gm , "");
	if (graphresult() != 0)
		{
		printf(" Your monitor does not support graphics : VGA 640 X 480 X 16 ");
		getch();
		exit(1);
		}
	}

void display_class::end_graphics()
	{
	if (mouse_present) mouse.conceal();
	closegraph();
	}

void display_class::set_line(int lineno, char* str)
	{
	if (mouse_present) mouse.conceal();

	if (!cleared[lineno]) clear_line(lineno);

	setcolor(talk_col[lineno]);
	settextstyle(TRIPLEX_FONT, 0, size[lineno] );
	setusercharsize(4, 8, 4, 8);

	outtextxy(x1[lineno], y1[lineno], str);

	cleared[lineno] = false;

	if (mouse_present) mouse.display();
	}

void display_class::clear_line (int lineno)
	{
	if (mouse_present) mouse.conceal();

	setfillstyle(SOLID_FILL, back_col);
	bar(x1[lineno], y1[lineno], x2[lineno], y2[lineno]);
	cleared[lineno] = true;

	if (mouse_present) mouse.display();
	}

void display_class::show_score(score num)
	{
	char str[20];
	sprintf(str, "Score: %ld", num);
	set_line(3, str);
	}

void display_class::show_time(char* p1, long int t1, char* p2, long int t2)
	{
	gotoxy(1, 6); printf("CLOCKS: ");
	gotoxy(1, 7); printf("%s: %ld secs", p1, t1);
	gotoxy(1, 8); printf("%s: %ld secs", p2, t2);
	}

void display_class::show_num_of_pos_moves(int t)
	{
	gotoxy(1, 10); printf("Possible Moves=%d  ",t);
	}

void display_class::show_list_of_moves(movetype m)
	{
	char str[5];

	settextstyle(DEFAULT_FONT, 0, 0);
	setcolor(7);

	int x = 585 + (num_of_moves%2) * 30, y = 10 + (num_of_moves/2) * 13;

	if (x==585)
		{
		sprintf(str, "%d.", (num_of_moves/2) + 1);
		outtextxy(560, y, str);
		}

	if (m==-1) sprintf(str, "-"); else sprintf(str, "%c%c", 'A'+m%8, '1'+m/8);
	outtextxy(x, y, str);

	num_of_moves++;
	}

void display_class::show_pos_move(pos place)
	{
	if (!indi_pos_moves) return;

	if (mouse_present) mouse.conceal();

	int x = grd_x + (place%8) * cell_wdth + cell_wdth/2;
	int y = grd_y - (place/8) * cell_hgt - cell_hgt/2;

	setcolor(YELLOW);
	setfillstyle(SOLID_FILL, YELLOW);
	bar(x-3,y-3,x+3,y+3);

	if (mouse_present) mouse.display();
	}

void display_class::unshow_pos_move(pos place)
	{
	if (!indi_pos_moves) return;

	if (mouse_present) mouse.conceal();

	int x = grd_x + (place%8) * cell_wdth + cell_wdth/2;
	int y = grd_y - (place/8) * cell_hgt - cell_hgt/2;

	setcolor(grd_back_col);
	setfillstyle(SOLID_FILL, grd_back_col);
	bar(x-3,y-3,x+3,y+3);

	if (mouse_present) mouse.display();
	}

void display_class::display_grid()
	{
	if (mouse_present) mouse.conceal();

	cleared[0] = cleared[1] = cleared[2] = cleared[3] = true;

	char str[10];

	setbkcolor(back_col);
	cleardevice();

	setfillstyle(SOLID_FILL, grd_back_col);
	bar(grd_x, grd_y, grd_x+cell_wdth*8, grd_y-cell_hgt*8);

	setcolor(grd_col);
	settextstyle(0,0,1);
	for (int i=0; i<=8; i++)
		{
		line(grd_x+i*cell_wdth, grd_y,
								grd_x+i*cell_wdth, grd_y-cell_hgt*8);
		line(grd_x, grd_y-i*cell_hgt,
									grd_x+ cell_wdth*8,grd_y-i*cell_hgt);


		str[0] = i+'A'; str[1] = 0;
		if (i!=8)
			outtextxy( grd_x+i*cell_wdth+cell_wdth/2 - 5, grd_y+5, str);

		str[0] = i+'1'; str[1]=0;
		if (i!=8)
			outtextxy( grd_x-9, grd_y-i*cell_hgt-cell_hgt/2 - 5, str);

		}

	settextstyle(GOTHIC_FONT,HORIZ_DIR, 0);
	setusercharsize(1,1,1,1);
	int x = 75;
	int y = 10;
	outtextxy(x, y, "OthellO");

	if (mouse_present) mouse.display();
	}

void display_class::show_piece(pos place, id who)
	{
	if (mouse_present) mouse.conceal();

	int x = grd_x + (place%8) * cell_wdth + cell_wdth/2;
	int y = grd_y - (place/8) * cell_hgt - cell_hgt/2;

	color col = ( (who==0)? grd_back_col : piece_cols[who] );

	setcolor(col);
	setfillstyle(SOLID_FILL, col);
	fillellipse(x, y, piece_rad, piece_rad);

	if (mouse_present) mouse.display();
	}

void display_class::turn_piece_to(pos place, id who)
	{
	if (show_turning)
	{

	if (mouse_present) mouse.conceal();

	int x = grd_x + (place%8) * cell_wdth + cell_wdth/2;
	int y = grd_y - (place/8) * cell_hgt - cell_hgt/2;

	color col1 = piece_cols[3-who], col2 = piece_cols[who];

	float theta, inc_theta = .2;
	int d=6;
	int time_del=40;
	for (theta = 0; theta <= 3.14159; theta += inc_theta)
		{
		int nd = d*sin(theta);
		int nr = abs(int(piece_rad*cos(theta)));

		if (theta <= 3.14159/2)
			{
			setcolor(col2);
			setfillstyle(SOLID_FILL, col2);
			ellipse(x-nd, y, 0, 360, nr, piece_rad);
			ellipse(x, y, 0, 360, nr, piece_rad);
			line (x-nd, y+piece_rad, x, y+piece_rad);
			line (x-nd, y-piece_rad, x, y-piece_rad);
			if (nd>=2) floodfill(x-nd-nr+1, y, col2);

			setcolor(col1);
			setfillstyle(SOLID_FILL, col1);
			ellipse(x, y, 0, 360, nr, piece_rad);
			fillellipse(x+nd, y, nr, piece_rad);
			line (x, y+piece_rad, x+nd, y+piece_rad);
			line (x, y-piece_rad, x+nd, y-piece_rad);
			if (nd>=2) floodfill(x-nr+1, y, col1);

			}
		else
			{
			setcolor(col1);
			setfillstyle(SOLID_FILL, col1);
			ellipse(x+nd, y, 0, 360, nr, piece_rad);
			ellipse(x, y, 0, 360, nr, piece_rad);
			line (x+nd, y+piece_rad, x, y+piece_rad);
			line (x+nd, y-piece_rad, x, y-piece_rad);
			if (nd>=2) floodfill(x+nd+nr-1, y, col1);

			setcolor(col2);
			setfillstyle(SOLID_FILL, col2);
			ellipse(x, y, 0, 360, nr, piece_rad);
			fillellipse(x-nd, y, nr, piece_rad);
			line (x, y+piece_rad, x-nd, y+piece_rad);
			line (x, y-piece_rad, x-nd, y-piece_rad);
			if (nd>=2) floodfill(x+nr-1, y, col2);

			}

		delay(time_del);

		setcolor(grd_back_col);
		setfillstyle(SOLID_FILL, grd_back_col);
		bar(x-cell_wdth/2+1, y-cell_hgt/2+1,x+cell_wdth/2-1, y+cell_hgt/2-1);
		}
	if (mouse_present) mouse.display();

	}

	show_piece(place, who);

	}

int display_class::get_loc(int x, int y)
	{
	if (x<grd_x || x>(grd_x+cell_wdth*8)
		||y>grd_y || x<(grd_y-cell_hgt*8))
			return -1;

	return ( (x - grd_x) / cell_wdth ) + ( (grd_y - y) / cell_hgt ) * 8;
	}

//class board member functions
board_class::board_class()
	{
	turn_delay = 50;
	turn_snd_frq = 500;
	}

void board_class::reset_board()
	{
	for( int i =0; i <= 63; i++)
		board[i] = 0;

	board[28] = board[35] = 2;
	board[27] = board[36] = 1;
	disp.show_piece(28, 2);
	disp.show_piece(27, 1);
	disp.show_piece(36, 1);
	disp.show_piece(35, 2);
	}

bool board_class::is_legal(id who, movetype m)
	{
	if (board[m]) return false;

	int r = m/8, c = m%8;
	id opp = 3 - who;

	if (c<=5)
		if (board[m+1]==opp)
			for ( int i=m+2; i%8!=0; i++)
				if (board[i]==who) return true;
				else if (board[i]==0) break;
	if (c>=2)
		if (board[m-1]==opp)
			for ( int i=m-2; i%8!=7 && i!=-1; i--)
				if (board[i]==who) return true;
				else if (board[i]==0) break;
	if (r>=2)
		if (board[m-8]==opp)
			for ( int i=m-16; i>=0 ; i-=8)
				if (board[i]==who) return true;
				else if (board[i]==0) break;
	if (r<=5)
		if (board[m+8]==opp)
			for ( int i=m+16; i<=63 ; i+=8)
				if (board[i]==who) return true;
				else if (board[i]==0) break;
	if (r<=5 && c<=5)
		if (board[m+9]==opp)
			for ( int i=m+18; (i<=63) && ((i%8)!=0) ; i+=9)
				if (board[i]==who) return true;
				else if (board[i]==0) break;
	if (r<=5 && c>=2)
		if (board[m+7]==opp)
			for ( int i=m+14; (i<=63) && ((i%8)!=7) ; i+=7)
				if (board[i]==who) return true;
				else if (board[i]==0) break;
	if (r>=2 && c<=5)
		if (board[m-7]==opp)
			for ( int i=m-14; i>=0 && i%8 != 0 ; i-=7)
				if (board[i]==who) return true;
				else if (board[i]==0) break;
	if (r>=2 && c>=2)
		if (board[m-9]==opp)
			for ( int i=m-18; (i>=0) && ((i%8)!=7) ; i-=9)
				if (board[i]==who) return true;
				else if (board[i]==0) break;

	return false;
	}

bool board_class::is_filled()
	{
	for(int i=0; i<=63; i++)
		if (board[i] == 0) return false;
	return true;
	}

int board_class::no_of_legal_moves(id who)
	{
	int to_return=0;

	for(int i=0; i<=63; i++)
		if (is_legal(who, i)) to_return++;

	return to_return;
	}

void board_class::play_move(id who, movetype m )
	{
	assert( is_legal(who, m) ); //***DEBUG

	board[m] = who;
	disp.show_piece(m, who);

	int r = m/8, c = m%8;
	id opp = 3 - who;

	if (c<=5)
		if (board[m+1]==opp)
			for ( int i=m+2; i%8!=0; i++)
				{
				if (board[i] == opp) continue;
				if (board[i]==who)
					for (int j=m+1; j!=i ; j++)
						{
						board[j] = who;
						disp.turn_piece_to(j, who);
						if (sound_output) sound(turn_snd_frq); delay(turn_delay); nosound();
						}
				break;  //do nothing for blank space
				}
	if (c>=2)
		if (board[m-1]==opp)
			for ( int i=m-2; i%8!=7 && i!=-1; i--)
				{
				if (board[i] == opp) continue;
				if (board[i]==who)
					for (int j=m-1; j!=i; j--)
						{
						board[j] = who;
						disp.turn_piece_to(j, who);
						if (sound_output) sound(turn_snd_frq); delay(turn_delay); nosound();
						}
				break;
				}
	if (r>=2)
		if (board[m-8]==opp)
			for ( int i=m-16; i>=0 ; i-=8)
				{
				if (board[i] == opp) continue;
				if (board[i]==who)
					for (int j=m-8; j!=i; j-=8)
						{
						board[j] = who;
						disp.turn_piece_to(j, who);
						if (sound_output) sound(turn_snd_frq); delay(turn_delay); nosound();
						}
				break;
				}
	if (r<=5)
		if (board[m+8]==opp)
			for ( int i=m+16; i<=63 ; i+=8)
				{
				if (board[i] == opp) continue;
				if (board[i]==who)
					for (int j=m+8; j!=i; j+=8)
						{
						board[j] = who;
						disp.turn_piece_to(j, who);
						if (sound_output) sound(turn_snd_frq); delay(turn_delay); nosound();
						}
				break;
				}
	if (r<=5 && c<=5)
		if (board[m+9]==opp)
			for ( int i=m+18; (i<=63) && ((i%8)!=0) ; i+=9)
				{
				if (board[i] == opp) continue;
				if (board[i]==who)
					for (int j=m+9; j!=i; j+=9)
						{
						board[j] = who;
						disp.turn_piece_to(j, who);
						if (sound_output) sound(turn_snd_frq); delay(turn_delay); nosound();
						}
				break;
				}
	if (r<=5 && c>=2)
		if (board[m+7]==opp)
			for ( int i=m+14; (i<=63) && ((i%8)!=7) ; i+=7)
				{
				if (board[i] == opp) continue;
				if (board[i]==who)
					for (int j=m+7; j!=i; j+=7)
						{
						board[j] = who;
						disp.turn_piece_to(j, who);
						if (sound_output) sound(turn_snd_frq); delay(turn_delay); nosound();
						}
				break;
				}
	if (r>=2 && c<=5)
		if (board[m-7]==opp)
			for ( int i=m-14; i>=0 && i%8 != 0 ; i-=7)
				{
				if (board[i] == opp) continue;
				if (board[i]==who)
					for (int j=m-7; j!=i; j-=7)
						{
						board[j] = who;
						disp.turn_piece_to(j, who);
						if (sound_output) sound(turn_snd_frq); delay(turn_delay); nosound();
						}
				break;
				}
	if (r>=2 && c>=2)
		if (board[m-9]==opp)
			for ( int i=m-18; (i>=0) && ((i%8)!=7) ; i-=9)
				{
				if (board[i] == opp) continue;
				if (board[i]==who)
					for (int j=m-9; j!=i ; j-=9)
						{
						board[j] = who;
						disp.turn_piece_to(j, who);
						if (sound_output) sound(turn_snd_frq); delay(turn_delay); nosound();
						}
				break;
				}
	}

int board_class::no_of_pieces(id who)
	{
	int to_return=0;
	for (int i=0; i<=63; i++)
		to_return += ( (board[i]==who)? 1 : 0 );
	return to_return;
	}

//class game member functions

game_class::~game_class()
	{
	player[1]. die();
	player[2]. die();
	}

void game_class::intro()
	{
	char ch;

	setbkcolor(0);
	cleardevice();

	settextstyle(TRIPLEX_FONT, 0, 0);
	setusercharsize(2,1,2,1);
	setcolor(YELLOW);
	outtextxy(320 - textwidth("Othello")/2, 50, "Othello");

	settextstyle(DEFAULT_FONT, 0, 1);
	outtextxy(320 - textwidth("Version 2.0")/2, 120, "Version 2.0");

	setcolor(CYAN);
	settextstyle(TRIPLEX_FONT, 0, 0);
	setusercharsize(5,8,5,8);
	outtextxy(320 - textwidth(
		"Programmed by: Gaurang Khetan, grkhetan@hotmail.com")/2,
		150,  "Programmed by: Gaurang Khetan, grkhetan@hotmail.com");

	int music1[] = {  500, 150, 3, 250, 700, 0, 1, 300,
							500, 150, 3, 250, 700, 0, 1, 300,
							500, 250, 2, 250, 1000, -100, 3, 200,
							0  };
	play_music(music1);

	gotoxy(1, 15);
	for (int i=1; i<=2; i++)
		{
		printf("\n Player %d Human/Computer ?", i);

		do ch = getch(); while (ch!='c' && ch!='C' && ch!= 'H' && ch!= 'h');
		putchar(ch);

		if (ch=='c' || ch=='C')  //computer
			{
			printf("\n Computer GamePlay Parameters: ");

			int depth, eg_empty, shallow_depth;
			do
				{
				printf("\n   Search Depth (4 to 16, typically 6): ");
				scanf("%d", &depth);
				}
			while (depth<4 || depth>16);
			do
				{
				printf("\n   Endgame Search with Empty Squares as(4 to 20, typically 10): ");
				scanf("%d", &eg_empty);
				}
			while (eg_empty<4 || eg_empty>20);
			do
				{
				printf("\n   Move Ordering Search Depth (2 to 4, typically 2): ");
				scanf("%d", &shallow_depth);
				}
			while (shallow_depth<2 || shallow_depth>4);

			player[i].set_nature(computer, i, depth, eg_empty, shallow_depth);
			}
		else  //human
			{
			player[i].set_nature(human,i,0,0,0);
			printf("\n Your name: ");

			int count = 0;
			char ch;
			char name[13];
			while(1)
				{
				ch = getch();
				if (ch==13 || count==12)
					{
					name[count] = 0;
					if (count==0) printf("Human");
					player[i].set_name(  (count==0)? "Human" : name  );
					break;
					}
				if (ch==8)
					{
					if (count==0) continue;
					count--;
					gotoxy(wherex()-1, wherey()); putchar(' ');
					gotoxy(wherex()-1, wherey());
					continue;
					}
				name[count] = ch;
				putchar(ch);
				count++;
				}
			}
		}

	printf("\n Show turning of pieces?(Y/N)");
	do ch = getch(); while (ch!='n' && ch!='N' && ch!= 'Y' && ch!= 'y');
	disp.set_turning( (ch=='y' || ch=='Y')? true : false );

	printf("\n Indicate possible moves?(Y/N)");
	do ch = getch(); while (ch!='n' && ch!='N' && ch!= 'Y' && ch!= 'y');
	disp.set_indi_pos_moves( (ch=='y' || ch=='Y')? true : false );

	}

id game_class::play_game()
	{

	id cur_player = 1;
	disp.show_time(player[1].get_name(), player[1].get_elap_time(),
							player[2].get_name(), player[2].get_elap_time() );
	disp.set_num_moves(0);

	while(1)
		{

		if (board.no_of_legal_moves(1)==0
			&& board.no_of_legal_moves(2)==0) break;

		if (kbhit() && getch()==27) { disp.end_graphics(); exit(1); }

		//get and play move
		movetype move = player[cur_player].get_move();
		if (move!=-1)
			board.play_move(cur_player, move);

		//update screen time and list of moves
		disp.show_time(player[1].get_name(), player[1].get_elap_time(),
							player[2].get_name(), player[2].get_elap_time() );
		disp.show_list_of_moves(move);

		//check for end of game


		//switch current player
		cur_player = 3 - cur_player;

		}

	//decide winner and show comment
	id winner;
	int t[3];
	t[1] = board.no_of_pieces(1); t[2] = board.no_of_pieces(2);

	if (t[1]>t[2]) winner = 1;
	else if(t[1]<t[2]) winner = 2;
	else winner = 0;

	char str1[100], str2[100];
	if (winner==0)
		{
		sprintf( str1, "Game Drawn. ");
		sprintf( str2, "Both with %d pieces.", t[1]);
		}
	else
		{
		sprintf( str1, "%s wins!!", player[winner].get_name() );
		sprintf( str2, "By %d-%d=%d pieces!!", t[winner], t[3-winner],
																t[winner] - t[3-winner]);
		}

	disp.set_line(1, str1);
	disp.set_line(2, str2);
	if (sound_output) sound(3000); delay(500); nosound();

	return winner;
	}

// player_class member functions
player_class::player_class()
	{
	elap_time=0;
	}

void player_class::die()
	{
	if (nature != human) delete brain;
	}

void player_class::set_name(char* str)
	{
	strncpy (name, str, 12);
	name[12] = 0;
	}

void player_class::set_nature(playertype p, id num, int depth, int eg_empty, int shallow_depth)
	{
	nature = p;
	plnumber = num;
	if(p==human) return;

	brain = new TheBrain10;

	if (brain==0)
		{
		printf("Sorry! Insufficient Memory for making the Computer Play!");
		if (sound_output) sound(1000); delay(100); nosound();
		getch();
		exit(1);
		}
	else if (brain->is_mem_err())
		{
		printf("Sorry! Insufficient Memory for making the Computer Play!");
		if (sound_output) sound(1000); delay(100); nosound();
		getch();
		exit(1);
		}

	brain->set_id(num, 3-num, 0);
	brain->set_depth(depth);	//set depth to be searched
	brain->set_endgame_empty(eg_empty);	//set the start of endgame with d squares empty
	brain->set_move_order_depth(shallow_depth);
	sprintf(name, "Computer-%d", depth);

	}

char* player_class::get_name()
	{
	return name;
	}

long int player_class::get_elap_time()
	{
	return elap_time;
	}

movetype player_class::get_move()
	{

	char str[100];
	sprintf(str, "Player %d to play: %s", plnumber, name);
	disp.set_line(2, str);

	int pos_moves[40];
	int num_of_pos_moves = 0;

	//indicate possible moves
	for(int i=0; i<=63; i++)
		if ( board.is_legal(plnumber, i) )
			{
			if (nature==human) disp.show_pos_move(i);
			pos_moves[num_of_pos_moves] = i;
			num_of_pos_moves++;
			}
	pos_moves[num_of_pos_moves] = 0;

	int t = num_of_pos_moves;  //board.no_of_legal_moves(plnumber);
	disp.show_num_of_pos_moves(t);

	if (t == 0)
		{
		sprintf(str, "%s has NO legal moves!", name);
		disp.set_line(1, str);
		disp.set_line(0, "Press a key");
		if (sound_output) sound(1000); delay(200); nosound();

		if (nature==human)
			getch();
		else
			{
			flushall();
			clock_t start;
			start = clock();
			do
				if (kbhit()) break;
			while ( ((clock()-start)/CLK_TCK) < 5 );
			}

		sprintf(str, "%s could not play", name);
		disp.set_line(1, str);
		disp.clear_line(0);

		return -1;
		}

	movetype move;

	time_t time1, time2;
	time(&time1);   //start timing

	if (nature==human)
		move = get_human_move();
	else
		move = get_comp_move();

	time(&time2);	//end timing
	elap_time += difftime(time2, time1);

	//put off indication of possible moves
	if (nature==human)
		for(int j=0; j<num_of_pos_moves; j++)
			disp.unshow_pos_move(pos_moves[j]);

	sprintf(str, "%s played at %c%c", name, move%8+'A', move/8 + '1');
	disp.set_line(1, str);

	return move;
	}

movetype player_class::get_comp_move()
	{
	char str[50];
	timeb t1, t2;

	sprintf(str, "%s thinking ...", name);
	disp.set_line(0, str);

	ftime(&t1);
	movetype move = brain->make_move(board.get_board());
	ftime(&t2);

	assert(board.is_legal(plnumber, move));

	disp.clear_line(0);

	disp.show_score(brain->get_cur_score());

	gotoxy(20,1);
	long int p = brain->get_leaves_evaluated();
	long int q = (t2.time - t1.time)*1000L + t2.millitm-t1.millitm;
	printf("Nodes: %ld %ld %6.2fK      ", p, q, (float)p/q);

	return move;
	}

movetype player_class::get_human_move()
	{
	char str[100];
	movetype move;

	if (mouse_present)
		{
		mouse.flush();

		disp.set_line(0, "Click where you want to play");

		while(1)
			{
			if ( kbhit() && getch()==27)
				{
				disp.end_graphics();
				exit(1);
				}

			if (mouse.was_button_pressed(leftbutton))
				{
				move = disp.get_loc ( mouse.get_x(), mouse.get_y() );
				if (move==-1) continue;
				if ( ! (board.is_legal(plnumber, move)) ) continue;
				break;
				}
			}
		}
	else
		{
		disp.set_line(0, "Type in coordinates of position");
		char ch;
		do
			{
			label0:

			while(1)
				{
				ch = getch();
				if (ch==27) { disp.end_graphics(); exit(0); }
				if (ch>='a' && ch<='h') { ch-=32; break; }
				if (ch>='A' && ch<='H') break;
				}
			move = ch-'A';

			str[0] = ch; str[1] = 0;
			disp.set_line(0, str);

			do
				{
				ch = getch();
				if (ch==27) { disp.clear_line(0);goto label0; }
				}
			while ( !(ch>='1' && ch<='8') );
			move += (8 * (ch-'1'));

			disp.clear_line(0);
			}
		while ( !(board.is_legal(plnumber, move)) );
		}
	return move;
	}

void play_music(int* music)
	{
	if (!sound_output) return;

	int step=0;
	while(music[step])
		{
		int start = music[step];
		int inc = music[step+1];
		int times = music[step+2];
		int duration = music[step+3];
		step+=4;

		for(int z = 1; z <= times; z++)
			{
			sound(start + inc*z);
			delay(duration);
			nosound();
			}
		}

	}
